%% Lattice Structure Generation and Evaluation Script (Last edited: 25 Jul 20)
function Main
warning('off');

%% Inputs

% Set overall lattice structure dimensions (mm). 
L = 120;
B = 80;
H = 50;

% Select lattice structure concept to generate.
Concept = 2; % Cube (1), Hexagonal Prism (2), Elongated Tetrakaidecahedron (3)

% Set strut radius (mm)
r = 1;

% Set unit cell inputs (mm unless specified otherwise).
% Cube
a = 50;

% Hexagonal Prism
p = 20;
q = 50;

% Elongated Tetrakaidecahedron
x = 10;
y = 12;
theta = 60; % degrees

%% Concept setup

% Set lattice structure files location.
filePath = 'E:\LatticeApp\';

% Open CATIA.
catia = actxserver('catia.application');
set(catia,'visible',1);

if Concept == 1 % Cube
    DesignInputs = [a, r, L, B, H];
    xlswrite(strcat(filePath,'Lattice\CubeDesignTable.xlsx'),DesignInputs,'A2:E2');
    catpart = strcat(filePath,'Lattice\Cube Lattice (Generative Shape Design).CATPart'); 
    RunAnalysis = strcat(filePath,'Lattice\MacroDirectory\ComputeCubeLattice.CATScript');
    DeflectionFile = strcat(filePath,'Lattice\Cube Displacement Values.txt');   
    
elseif Concept == 2 % Hexagonal Prism    
    DesignInputs = [p, q, r, L, B, H];
    xlswrite(strcat(filePath,'Lattice\HexagonalPrismDesignTable.xlsx'),DesignInputs,'A2:F2');
    catpart = strcat(filePath,'Lattice\Hexagonal Prism Lattice (Generative Shape Design).CATPart');
    RunAnalysis = strcat(filePath,'Lattice\MacroDirectory\ComputeHexagonalPrismLattice.CATScript');
    DeflectionFile = strcat(filePath,'Lattice\Hexagonal Prism Displacement Values.txt');    
    
elseif Concept == 3 % Elongated Tetrakaidecahedron 
    DesignInputs = [x, y, theta, r, L, B, H];
    xlswrite(strcat(filePath,'Lattice\ElongatedTetrakaidecahedronDesignTable.xlsx'),DesignInputs,'A2:G2');
    catpart = strcat(filePath,'Lattice\Elongated Tetrakaidecahedron Lattice.CATPart');
    RunAnalysis = strcat(filePath,'Lattice\MacroDirectory\ComputeElongatedTetrakaidecahedronLattice.CATScript');
    DeflectionFile = strcat(filePath,'Lattice\Elongated Tetrakaidecahedron Displacement Values.txt');
    
end

%% Generate and Evaluate

% Load model file.
invoke(get(catia,'Documents'),'Open',catpart);

try
    system(RunAnalysis);
    PauseTime = 0;
    while exist(DeflectionFile,'file') == 0
        pause(1)
        PauseTime = PauseTime + 1;
        if PauseTime == 900 % Maximum allowable wait time (s). 
            break
        end    
    end

    % Read displacement results.
    pause(1)
    fileID = fopen(DeflectionFile,'r');
    DisplacementVector = textscan(fileID, '%s%s%s%s%s%s');
    fclose('all');
    delete(DeflectionFile);
    
    C1 = DisplacementVector{1,4};
    C1(1:3,:) = [];
    C1 = str2double(C1);
    
    C2 = DisplacementVector{1,5};
    C2(1:3,:) = [];
    C2 = str2double(C2);
    
    C3 = DisplacementVector{1,6};
    C3(1:3,:) = [];
    C3 = str2double(C3);

    % Get max displacement. 
    maxDisplacement = max(sqrt(C1.^2 + C2.^2 + C3.^2));
    doc = get(catia,'activedocument');
    invoke(doc,'save');
    invoke(doc,'close');
    
    % Get volume and surface area. 
    doc = get(catia,'activedocument');
    product = get(doc,'product');
    volume = get(get(product,'Analyze'),'Volume');
    area = get(get(product,'Analyze'),'WetArea');
    invoke(doc,'save');
    invoke(doc,'close');         
catch        
    warning('Invalid geometry. Change unit cell parameters.') 
    volume = 10e6;
    area = 10e-6;
    maxDisplacement = 10e6;        
end

% Close CATIA
invoke(catia,'quit');
invoke(catia,'delete');

% Display objective values.
fprintf('Volume (mm^3) = %.3f\n',volume)
fprintf('Surface Area (mm^2) = %.3f\n',area)
fprintf('Maximum Displacement (mm) = %.6f\n',maxDisplacement)

%% Constraints
% Cube
% L > a
% B > a
% H > a
% 
% Hexagonal Prism
% L > 2*p*cos(pi/6)
% B > 2*p*cos(pi/6)
% H > q
% 
% Elongated Tetrakaidecahedron
% Z = 4*y*sin(theta*pi/180)
% W = 2*y*cos(theta*pi/180) + sqrt(2)*x
% x/y > 0
% x/y < 2*sqrt(2)
% theta < 90
% theta > (180/pi) * asin(sqrt(2)*x/(5*y) + sqrt(2)/10*sqrt(10 - x^2/y^2)
% L > W
% B > W
% H > Z

return
